/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "popupmessage.h"


#include <qtextbrowser.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qframe.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qpixmap.h>

#include <config.h>

PopupMessage::PopupMessage( QWidget *parent, QWidget *anchor, int timeout, const char *name )
: OverlayWidget( parent, anchor, name )
, m_anchor( anchor )
, m_parent( parent )
, m_offset( 0 )
, m_counter( 0 )
, m_stage( 1 )
, m_timeout( timeout )
, m_showCounter( true )
{
	QHBoxLayout *hbox;
	QLabel *label;
	QTextBrowser *alabel;
	setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Preferred );

	setFrameStyle( QFrame::Panel | QFrame::Raised );
	setFrameShape( QFrame::StyledPanel );
	setWFlags( Qt::WX11BypassWM );


	m_layout = new QVBoxLayout( this, 9, 6 );
	
	hbox = new QHBoxLayout( m_layout, 12);
	
	hbox->addWidget( m_countdownFrame = new QFrame( this, "counterVisual" ) );
	m_countdownFrame->setFixedWidth( fontMetrics().width( "X" ) );
	m_countdownFrame->setFrameStyle( QFrame::Plain | QFrame::Box );
	m_countdownFrame->setPaletteForegroundColor( paletteBackgroundColor().dark() );

	alabel = new QTextBrowser( this, "label");
	alabel->setTextFormat( Qt::RichText );
	alabel->setPaper( parentWidget()->backgroundBrush());
	alabel->setVScrollBarMode( QScrollView::AlwaysOff);
	alabel->setHScrollBarMode( QScrollView::AlwaysOff);
	//alabel->setWordWrap ( QTextEdit::NoWrap);
	alabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	alabel->setFrameShape( QFrame::StyledPanel );
	alabel->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Preferred );

	hbox->add( alabel );

	label = new QLabel( this, "image");
	hbox->add( label);

	QPushButton *closeButton = new QPushButton( this, "closeButton");
	closeButton->setPixmap( QPixmap( "no.png"));
	hbox->add( closeButton);
	connect( closeButton, SIGNAL(clicked()), this, SLOT(close()) );
}

void PopupMessage::addWidget( QWidget *widget )
{
	m_layout->add( widget );
	adjustSize();
}

void PopupMessage::showCloseButton( const bool show )
{
	static_cast<QPushButton*>(child( "closeButton" ))->setShown( show );
	adjustSize();
}

void PopupMessage::showCounter( const bool show )
{
	m_showCounter = show;
	static_cast<QFrame*>(child( "counterVisual" ))->setShown( show );
	adjustSize();
}

void PopupMessage::setText( const QString &text )
{
	static_cast<QTextBrowser *>(child( "label" ))->setText( text );
	adjustSize();
}

void PopupMessage::setImage( const QString &location )
{
	static_cast<QLabel*>(child( "image" ))->setPixmap( QPixmap( location ) );
	adjustSize();
}

void PopupMessage::setImage( const QPixmap &pix )
{
	static_cast<QLabel*>(child( "image" ))->setPixmap( pix );
	adjustSize();
}


////////////////////////////////////////////////////////////////////////
//     Public Slots
////////////////////////////////////////////////////////////////////////

void PopupMessage::close() //SLOT
{
	m_stage = 3;
	killTimer( m_timerId );
	m_timerId = startTimer( 6 );
}

void PopupMessage::display() //SLOT
{
	m_timerId = startTimer( 6 );
	show();
}

////////////////////////////////////////////////////////////////////////
//     Protected
////////////////////////////////////////////////////////////////////////

void PopupMessage::timerEvent( QTimerEvent* )
{
	slideMask();	
}

void PopupMessage::countDown()
{
	if( !m_timeout ) {
		killTimer( m_timerId );
		return;
	}

	QFrame *&h = m_countdownFrame;

	if( m_counter < h->height() - 3 )
		QPainter( h ).fillRect( 2, 2, h->width() - 4, m_counter, palette().active().highlight() );
	
	if( !hasMouse() )
		m_counter++;
	
	if( m_counter > h->height()) {
		m_stage = 3;
		killTimer( m_timerId);
		m_timerId = startTimer( 6);
	}
	else {
		killTimer( m_timerId );
		m_timerId = startTimer( m_timeout / h->height() );
	}
}


void PopupMessage::slideMask()
{
	switch( m_stage ) {

		case 1: //raise
			move( 0, m_parent->y() - m_offset );
			m_offset++;
			if( m_offset > height() ) {
				killTimer( m_timerId );
	
				if( m_timeout ) {
				m_timerId = startTimer( 40 );
				m_stage = 2;
				}
			}
			break;
	
		case 2: //fill in pause timer bar
			countDown();
			break;
	
		case 3: //lower
			m_offset--;
			move( 0, m_parent->y() - m_offset );
	
			if( m_offset < 0 ) {
			//Stop and restart
				killTimer( m_timerId );
				m_stage = 1;
				m_offset = 0;
				m_counter = 0;
				hide();
				QTimer::singleShot ( 500, this, SLOT( slotUpdateListMessage()));
			}
	}
}


void PopupMessage::displayMessage( const QString &text)
{
	m_list.append( MessageItem( text));
	if( m_list.count() == 1)
		nextMessage();
}

void PopupMessage::nextMessage()
{
	if( !m_list.count()) return;
	
	MessageItem item = m_list.first();
	
	raise();
	setText( item.m_text );
	reposition();
	display();
}

void PopupMessage::slotUpdateListMessage()
{
	m_list.pop_front();
	if( m_list.count() > 0)
		nextMessage();
}

//#ifdef CMAKE_BS
  #include "popupmessage.moc"
//#endif

