/***************************************************************************
 *   Copyright (C) 2006 by David Lacort Ankli   *
 *   david.lacort@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmggraphicobject.h"
#include <math.h>

NMGGraphicObject::NMGGraphicObject(QString qstr_name)
/**
\brief Create an instance without data
*/
{
	m_posl_vertexList.setAutoDelete(true);
	m_pos_maxValues.x = 0.0;
	m_pos_maxValues.y = 0.0;
	m_pos_minValues.x = 0.0;
	m_pos_minValues.y = 0.0;
	m_col_lineColor = Qt::black;
	m_col_areaColor = Qt::lightGray;
	m_b_hidden = false;
	m_b_average = true;
	m_b_accumulate = false;
	m_b_drawZeroIfNoData = false;
	m_nmvp_polygon = VPNONE;
	m_qstr_name = qstr_name;
	m_c_representation = 0x00;
	m_i_dotSize = 2;
}


NMGGraphicObject::~NMGGraphicObject()
/**
\brief Destroy the instance
*/
{
}

void NMGGraphicObject::setVertexList(QPtrList<double> &da_xData, QPtrList<double> &da_yData)
/**
\brief Set the graphic's position data
  
\param da_xData   The list of vertex positions' x values
\param da_yData   The list of vertex positions' y values
\param ui_newNumVertex   The list's number of vertex
*/
{
	// if the vertex list already exists, delete it
	if (!m_posl_vertexList.isEmpty())
		m_posl_vertexList.clear();
	
	if (da_yData.isEmpty())
		return;
	// copy the list
	uint i=0;
	m_pos_maxValues.x = 0.0;
	m_pos_maxValues.y = 0.0;
	double *d_x = da_xData.first();
	double *d_y = da_yData.first();
	m_pos_minValues.x = *d_x;
	m_pos_minValues.y = *d_y;
	while(d_y!=0)
	{
		NMPosition *nmp_newPos = new NMPosition();
		nmp_newPos->x = (d_x==0?(double)i:*d_x);
		nmp_newPos->y = (*d_y);
		m_posl_vertexList.append(nmp_newPos);
		
		m_pos_maxValues.x = fmax(m_pos_maxValues.x,nmp_newPos->x);
		m_pos_maxValues.y = fmax(m_pos_maxValues.y,nmp_newPos->y);
		m_pos_minValues.x = fmin(m_pos_minValues.x,nmp_newPos->x);
		m_pos_minValues.y = fmin(m_pos_minValues.y,nmp_newPos->y);

		d_x = da_xData.next();
		d_y = da_yData.next();
		i++;
	}
}

QString NMGGraphicObject::getName()
/**
  @brief Get the object name
  
  @return   Returns the object name
*/
{
	return m_qstr_name;
}

uint NMGGraphicObject::getNumVertex() 
/**
  @brief Get the vertex list's elements number
  
  @return   Returns the vertex list's elements number
*/
{
	return 	m_posl_vertexList.count();
}

bool NMGGraphicObject::first()
/**
	@brief Points the current item pointer to the first element

	@return	Returns false if the list is empty. 
*/
{
	return (m_posl_vertexList.first() != 0);
}

bool NMGGraphicObject::last()
/**
	@brief Points the current item pointer to the last element

	@return	Returns false if the list is empty. 
*/
{
	return (m_posl_vertexList.last() != 0);
}

bool NMGGraphicObject::next()
/**
	@brief Points the current item pointer to the next list element

	@return	Returns false if there are not next element (the current is the last one).
*/
{
	return (m_posl_vertexList.next() != 0);
}

bool NMGGraphicObject::prev()
/**
	@brief Points the current item pointer to the previous list element

	@return	Returns false if there are not previous element (the current is the first one).
*/
{
	return (m_posl_vertexList.prev() != 0);
}

double NMGGraphicObject::getCurrentX()
/**
  @brief Get the list's i_pos vertex's X coordinate
  
  @param i_pos   The vertex position in the list
  @return   Returns the list's i_pos vertex's X coordinate
*/
{
	return m_posl_vertexList.current()->x;
}
double NMGGraphicObject::getCurrentY()
/**
  @brief Get the list's i_pos vertex's Y coordinate
  
  @param i_pos   The vertex position in the list
  @return   Returns the list's i_pos vertex's Y coordinate
*/
{
	return m_posl_vertexList.current()->y;
}

double NMGGraphicObject::getMaxXValue()
/**
  @brief Get the list's maximum value in the X coordinate
  
  @return   Returns the list's maximum X coordinate value
*/
{
	return m_pos_maxValues.x;
}
double NMGGraphicObject::getMaxYValue()
/**
  @brief Get the list's maximum value in the Y coordinate
  
  @return   Returns the list's maximum Y coordinate value
*/
{
	return m_pos_maxValues.y;
}

double NMGGraphicObject::getMinXValue()
/**
  @brief Get the list's maximum value in the X coordinate
  
  @return   Returns the list's maximum X coordinate value
*/
{
	return m_pos_minValues.x;
}
double NMGGraphicObject::getMinYValue()
/**
  @brief Get the list's maximum value in the Y coordinate
  
  @return   Returns the list's maximum Y coordinate value
*/
{
	return m_pos_minValues.y;
}

void NMGGraphicObject::setHidden(bool b_hidden)
/**
		@brief Sets if this object is hidden or not
  
		@param b_hidden	True if it's hidden, false if not
 */
{
	m_b_hidden = b_hidden;
}

void NMGGraphicObject::setAverage(bool b_average)
/**
		@brief Sets if this graph is represented as an average or not
  
		@param b_average	True if wanted to show averages when there are more dots in the graph than pixels in the viewport
 */
{
	m_b_average = b_average;
	if (b_average)
		m_b_accumulate = false;
}

void NMGGraphicObject::setAccumulate(bool b_accumulate)
/**
		@brief Sets if this graph is represented as an accumulation or not
  
		@param b_accumulate	True if wanted to show accumulated values when there are more dots in the graph than pixels in the viewport
 */
{
	m_b_accumulate = b_accumulate;
	if (b_accumulate)
		m_b_average = false;
}

void NMGGraphicObject::setDrawZeroIfNoData(bool b_drawZero)
/**
		@brief Sets if a 0 value have to be painted if thre is no data in a X pixel
  
		@param b_drawZero	True if wanted to show 0 values when there are no data in teh current pixel in the viewport
 */
{
	m_b_drawZeroIfNoData = b_drawZero;
}

void NMGGraphicObject::setRepresentation(char c_representation)
/**
		@brief Sets the representation mode to be shown
  
		@param c_representation	The representation mode
 */
{
	m_c_representation = c_representation;
}

void NMGGraphicObject::setShowLines(bool b_show)
/**
		@brief Sets if lines between dots are show in the representation
  
		@param b_show	If lines between dots are shown
 */
{
	if (b_show)
		m_c_representation = (m_c_representation | NM_LINES);
	else	
		m_c_representation = (m_c_representation & (~NM_LINES));
}

void NMGGraphicObject::setShowArea(bool b_show)
/**
		@brief Sets if an area is shown under lines in the representation
  
		@param b_show	If area under lines is shown
 */
{
	if (b_show)
		m_c_representation = (m_c_representation | NM_AREA);
	else
		m_c_representation = (m_c_representation & (~NM_AREA));
}

void NMGGraphicObject::setShowPolygons(bool b_show)
/**
		@brief Sets if dots as polygons are shown in the representation
  
		@param b_show	If dots as polygons are shown
 */
{
	if (b_show)
		m_c_representation = (m_c_representation | NM_POLYGONS);
	else
		m_c_representation = (m_c_representation & (~NM_POLYGONS));
}

bool NMGGraphicObject::getHidden()
/**
		@brief To know if the graph is shown
  
		@return   Returns the hidden property
 */
{
	return m_b_hidden;
}

bool NMGGraphicObject::getAverage()
/**
		@brief To know if the graph is shown as an average graph
  
		@return   Returns the average property
 */
{
	return m_b_average;
}

bool NMGGraphicObject::getAccumulate()
/**
		@brief To know if the graph is shown as an accumulation values graph
  
		@return   Returns the accumulation property
 */
{
	return m_b_accumulate;
}

bool NMGGraphicObject::getDrawZeroIfNoData()
/**
		@brief To know if 0 values must be shown when no data in a pixel
  
		@return   Returns the 'draw zero if no data' property
 */
{
	return m_b_drawZeroIfNoData;
}

bool NMGGraphicObject::getShowLines()
/**
		@brief To know if lines between dots are shown
  
		@return   Returns true if lines between dots are shown, false otherwise
 */
{
	return ((m_c_representation & NM_LINES) > 0x00);
}

bool NMGGraphicObject::getShowArea()
/**
		@brief To know if an area under the lines graph is shown
  
		@return   Returns true if area is shown, false otherwise
 */
{
	return ((m_c_representation & NM_AREA) > 0x00);
}

bool NMGGraphicObject::getShowPolygons()
/**
		@brief To know if dots are represented as polygons
  
		@return   Returns true if dots are shown as polygons, false otherwise
 */
{
	return ((m_c_representation & NM_POLYGONS) > 0x00);
}



void NMGGraphicObject::setLineColor(QColor col_newColor)
/**
  @brief Set the graphic representation color attribute
  
  @param col_newColor   The representation's new color
*/
{
	m_col_lineColor = col_newColor;
}

void NMGGraphicObject::setAreaColor(QColor col_newColor)
/**
  @brief Set the graphic representation area color attribute
  
  @param col_newColor   The representation's new area color
*/
{
	m_col_areaColor = col_newColor;
}

QColor NMGGraphicObject::getLineColor()
/**
  @brief Get the graphic representation line color
  
  @return   Returns the graphic representation line color
*/
{
	return m_col_lineColor;
}

QColor NMGGraphicObject::getAreaColor()
/**
  @brief Get the graphic representation area color
  
  @return   Returns the graphic representation area color
*/
{
	return m_col_areaColor;
}

void NMGGraphicObject::setVertexPolygon(NMVertexPolygon nmvp_polygon)
/**
  @brief Set the polygon drawed in vertex in NMLINEPOLYGONS mode
  
  @param nmvp_polygon   The representation's new polygon
*/
{
	m_nmvp_polygon = nmvp_polygon;
}

NMGGraphicObject::NMVertexPolygon NMGGraphicObject::getVertexPolygon()
/**
  @brief Get the vertex polygon in NMLINEPOLYGONS mode
  
  @return   Returns the graphic representation flags
*/
{
	if ( m_c_representation & NM_POLYGONS == 0x00 )
		return VPNONE;
	else
		return m_nmvp_polygon;
}

void NMGGraphicObject::setDotsSize(int i_dotSize)
/**
  @brief Set the dot size in NMDOTS mode
  
  @param i_dotSize   The dot size in pixels
*/
{
	m_i_dotSize = i_dotSize;
}

int NMGGraphicObject::getDotsSize()
/**
  @brief Get the dot size showed in NMDOTS mode
  
  @return   Returns the dots size, in pixels
*/
{
	return m_i_dotSize;
}
